/**
   Our program IS a JFrame, so it no longer needs to
   instantiate a JFrame. This design occaisionally has
   an advantage,but in this simple case, it reallly
   doesn't do much to improve the design.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

@SuppressWarnings("serial")
public class InteractiveFrame_v10 extends JFrame
{
   public InteractiveFrame_v10()
   {
      super("Interactive Frame with FrameBuffer");
      FrameBuffer fb = new FrameBuffer(600, 600);
      FrameBufferPanel fbp = new FrameBufferPanel(fb);
      this.add(fbp);
      this.pack();
      this.setVisible(true);

      Scene scene = new Scene(); // Put scene in the scope of the KeyListener.

      this.addKeyListener(
         new KeyListener(){ // An anonymous inner class constructor.
            // Implement the three methods of the KeyListener interface.
            @Override public void keyPressed (KeyEvent e){}
            @Override public void keyReleased(KeyEvent e){}
            @Override public void keyTyped   (KeyEvent e)
            {
               System.out.println(e);
               for (Vertex v : scene.modelList.get(0).vertexList)
               {
                  v.z -= 0.2;
               }
               fb.clearFB();
               Pipeline.render(scene, fb.vp);
               fbp.update();
            }
         }
      );

      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }//constructor


   public static void main(String[] args)
   {
      new InteractiveFrame_v10(); // Weird!
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                                InteractiveFrame_v10 constructor
                                                activation record of final local
                                                variables
                                                +----------------------------+
                                                | final FrameBufferPanel fbp |
                                                | final Scene scene          |
InteractiveFrame_v10 object                     |                            |
(is a JFrame object)                            |                            |
+----------------------+                        +-----------/\---------------+
|                      |       KeyListener                  /\
|  List<KeyListener>---|------>>+------+                    ||  nesting link
|                      |        |      |                    ||   (closure)
|                      |        +------+        +-----------||------------+
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
