/**
   This program demonstrates a subtle race condition between
   a gui constructor and the Java gui event system. When the
   main() thread is delayed, but the Event Dispatch Thread
   isn't, the event handler sees a partialy constructed scene.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

@SuppressWarnings("serial")
public class InteractiveFrame_v11 extends JFrame
{
   public InteractiveFrame_v11()
   {
      super("Interactive Frame with FrameBuffer");
      FrameBuffer fb = new FrameBuffer(600, 600);
      FrameBufferPanel fbp = new FrameBufferPanel(fb);
      this.add(fbp);
      this.pack();
      this.setVisible(true);

      Scene scene = new Scene(); // Put scene in the scope of the KeyListener.

      this.addKeyListener(
         new KeyListener(){ // An anonymous inner class constructor.
            // Implement the three methods of the KeyListener interface.
            @Override public void keyPressed (KeyEvent e){}
            @Override public void keyReleased(KeyEvent e){}
            @Override public void keyTyped   (KeyEvent e)
            {
               System.out.println(e);
               for (Vertex v : scene.modelList.get(0).vertexList)
               {
                  v.z -= 0.2;
               }
               fb.clearFB();
               Pipeline.render(scene, fb.vp);
               fbp.update();
            }
         }
      );

      // Force a race condition by delaying this thread.
      try{Thread.sleep(2000);}catch(InterruptedException e){} // milliseconds

      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }//constructor


   public static void main(String[] args)
   {
      new InteractiveFrame_v11();
   }
}
