/**
   The important objects in our program that the listener
   object needs to access are now static variables in our
   program. This way the listener can see them (we are
   using a form of "global variables" which is NOT good).
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class InteractiveFrame_v2
{
   public static FrameBufferPanel fbp = null;
   public static Scene scene = null;

   public static void main(String[] args)
   {
      FrameBuffer fb = new FrameBuffer(600, 600);
      fbp = new FrameBufferPanel(fb);
      JFrame jf = new JFrame("Interactive Frame with FrameBuffer");
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);
      jf.addKeyListener(new KListener_v2());

      scene = new Scene();
      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }//main()
}


// A private class definition.
class KListener_v2 implements KeyListener
{
   @Override public void keyPressed (KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped   (KeyEvent e)
   {
      System.out.println(e);
      for (Vertex v : InteractiveFrame_v2.scene.modelList.get(0).vertexList)
      {
         v.z -= 0.2;
      }
      InteractiveFrame_v2.fbp.getFrameBuffer().clearFB();
      Pipeline.render(InteractiveFrame_v2.scene, InteractiveFrame_v2.fbp.getFrameBuffer().vp);
      InteractiveFrame_v2.fbp.update();
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                               InteractiveFrame_v2 class
                                               +===========================+
                                              / static FrameBufferPanel fbp \
                                              \ static Scene scene          /
                                              /                             \
      JFrame object                           \ staic main()                /
+----------------------+                       +===========================+
|                      |       KeyListener
|  List<KeyListener>---|------>>+------+
|                      |        |      |           KListener_v2 object
|                      |        +------+        +-------------------------+
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
